@@ -1,6 +1,6 @@
 <?php
 /* 
-V4.72 21 Feb 2006  (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights reserved.
+V4.80 8 Mar 2006  (c) 2000-2007 John Lim (jlim#natsoft.com.my). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence. 
@@ -9,7 +9,9 @@ V4.72 21 Feb 2006  (c) 2000-2006 John Lim (jlim@natsoft.com.my). All rights rese
   Latest version is available at http://adodb.sourceforge.net
 */
 
-error_reporting(E_ALL);
+
+if (!defined('E_STRICT')) define('E_STRICT',E_NOTICE);
+error_reporting(E_ALL|E_STRICT);
 
 $ADODB_FLUSH = true;
 
@@ -75,6 +77,8 @@ function testdb(&$db,$createtab="create table ADOXYZ (id int, firstname char(24)
 {
 GLOBAL $ADODB_vers,$ADODB_CACHE_DIR,$ADODB_FETCH_MODE,$ADODB_COUNTRECS;
 
+	//adodb_pr($db);
+	
 ?>	<form method=GET>
 	</p>
 	<table width=100% ><tr><td bgcolor=beige>&nbsp;</td></tr></table>
@@ -83,7 +87,7 @@ GLOBAL $ADODB_vers,$ADODB_CACHE_DIR,$ADODB_FETCH_MODE,$ADODB_COUNTRECS;
 	$create =false;
 	/*$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
 	
-	$rs = $db->Execute('select lastname,firstname,lastname,id from adoxyz');
+	$rs = $db->Execute('select lastname,firstname,lastname,id from ADOXYZ');
 	$arr = $rs->GetAssoc();
 	echo "<pre>";print_r($arr);
 	die();*/
@@ -122,13 +126,16 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 	if (defined('ADODB_EXTENSION')) $ext = ' &nbsp; Extension '.ADODB_EXTENSION.' installed';
 	else $ext = '';
 	print "<h3>ADODB Version: $ADODB_vers Host: <i>$db->host</i> &nbsp; Database: <i>$db->database</i> &nbsp; PHP: $phpv $ext</h3>";
+	
 	flush();
 
+	if (function_exists('date_default_timezone_set')) date_default_timezone_set('Asia/Kuala_Lumpur');
 	
 	$arr = $db->ServerInfo();
 	print_r($arr);
-	echo "<br>";
-	$e = error_reporting(E_ALL-E_WARNING);
+	echo E_ALL,' ',E_STRICT, "<br>";
+	$e = error_reporting(E_ALL | E_STRICT);
+	echo error_reporting(),'<p>';
 	flush();
 	#$db->debug=1;
 	$tt  = $db->Time(); 
@@ -162,11 +169,11 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 		print "<i>Error='$err'</i></p>";
 		if ($rez) print "<b>Cannot check if connection failed.</b> The Connect() function returned true.</p>";
 	}
-	error_reporting($e);
+	#error_reporting($e);
 	flush();
 
 	//$ADODB_COUNTRECS=false;
-	$rs=$db->Execute('select * from adoxyz order by id');
+	$rs=$db->Execute('select * from ADOXYZ order by id');
 	if($rs === false) $create = true;
 	else $rs->Close();
 	
@@ -178,24 +185,24 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 			return;
 		} else {
 			$db->debug = 99;
-			$e = error_reporting(E_ALL-E_WARNING);
+		#	$e = error_reporting(E_ALL-E_WARNING);
 			$db->Execute($createtab);
-			error_reporting($e);
+		#	error_reporting($e);
 		}
 	}
-	error_reporting(E_ALL);
+	#error_reporting(E_ALL);
 	echo "<p>Testing Metatypes</p>";
 	$t = $db->MetaType('varchar');
 	if ($t != 'C') Err("Bad Metatype for varchar");
 	
-	$rs = &$db->Execute("delete from ADOXYZ"); // some ODBC drivers will fail the drop so we delete
+	$rs = $db->Execute("delete from ADOXYZ"); // some ODBC drivers will fail the drop so we delete
 	if ($rs) {
 		if(! $rs->EOF) print "<b>Error: </b>RecordSet returned by Execute('delete...') should show EOF</p>";
 		$rs->Close();
 	} else print "err=".$db->ErrorMsg();
 
 	print "<p>Test select on empty table, FetchField when EOF, and GetInsertSQL</p>";
-	$rs = &$db->Execute("select id,firstname from ADOXYZ where id=9999");
+	$rs = $db->Execute("select id,firstname from ADOXYZ where id=9999");
 	if ($rs && !$rs->EOF) print "<b>Error: </b>RecordSet returned by Execute(select...') on empty table should show EOF</p>";
 	if ($rs->EOF && (($ox = $rs->FetchField(0)) && !empty($ox->name))) {
 		$record['id'] = 99;
@@ -219,10 +226,10 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 		$rs = $db->Execute("insert into ADOXYZ (id,firstname,lastname,created) values (99,'Should Not','Exist (Commit)',$time)");
 		if ($rs && $db->CommitTrans()) {
 			$rs->Close();
-			$rs = &$db->Execute("select * from ADOXYZ where id=99");
+			$rs = $db->Execute("select * from ADOXYZ where id=99");
 			if ($rs === false || $rs->EOF) {
 				print '<b>Data not saved</b></p>';
-				$rs = &$db->Execute("select * from ADOXYZ where id=99");
+				$rs = $db->Execute("select * from ADOXYZ where id=99");
 				print_r($rs);
 				die();
 			} else print 'OK</p>';
@@ -247,7 +254,7 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 		} else
 			print "<b>Commit failed</b></p>";
 			
-		$rs = &$db->Execute('delete from ADOXYZ where id>50');
+		$rs = $db->Execute('delete from ADOXYZ where id>50');
 		if ($rs) $rs->Close();
 		
 		if ($db->transCnt != 0) Err("Invalid transCnt = $db->transCnt (should be 0)");
@@ -318,7 +325,7 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 		$a = $db->MetaPrimaryKeys('ADOXYZ');
 		var_dump($a);
 	}
-	$rs = &$db->Execute('delete from ADOXYZ');
+	$rs = $db->Execute('delete from ADOXYZ');
 	if ($rs) $rs->Close();
 	
 	$db->debug = false;
@@ -342,7 +349,7 @@ FROM `nuke_stories` `t1`, `nuke_authors` `t2`, `nuke_stories_cat` `t3`, `nuke_to
 ef")."</p>";//'
 
 		print "<p>Testing Foreign Keys</p>";
-		$arr = $db->MetaForeignKeys('adoxyz',false,true);
+		$arr = $db->MetaForeignKeys('ADOXYZ',false,true);
 		print_r($arr);
 		if (!$arr) Err("No MetaForeignKeys");
 		break;
@@ -747,7 +754,14 @@ END Adodb;
 		where id=".$db->Param('zid')." and created>=".$db->Param('ZDATE')."",
 		$array);
 	if ($id != 1) Err("Bad bind; id=$id");
-	else echo "<br>Bind date/integer passed";
+	else echo "<br>Bind date/integer 1 passed";
+	
+	$array =array(1,$db->BindDate(time()));
+	$id = $db->GetOne("select id from ADOXYZ 
+		where id=".$db->Param('0')." and created>=".$db->Param('1')."",
+		$array);
+	if ($id != 1) Err("Bad bind; id=$id");
+	else echo "<br>Bind date/integer 2 passed";
 	
 	$db->debug = false;
 	$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
@@ -760,7 +774,7 @@ END Adodb;
 		print_r($rs->fields);
 	}
 	if ($db->databaseType !== 'odbc') {
-		$rs = &$db->Execute("select id,firstname,lastname,created,".$db->random." from ADOXYZ order by id");
+		$rs = $db->Execute("select id,firstname,lastname,created,".$db->random." from ADOXYZ order by id");
 		if ($rs) {
 			if ($rs->RecordCount() != 50) {
 				print "<p><b>RecordCount returns ".$rs->RecordCount().", should be 50</b></p>";
@@ -785,7 +799,7 @@ END Adodb;
 	 echo "<b>GetRow Test</b>";
 	$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
 	$val1 = $db->GetRow("select count(*) from ADOXYZ");
-	$val2 =& $db->GetRow("select count(*) from ADOXYZ");
+	$val2 = $db->GetRow("select count(*) from ADOXYZ");
 	 if ($val1[0] == 50 and sizeof($val1) == 1 and $val2[0] == 50 and sizeof($val2) == 1) print "<p>GetRow returns ok</p>";
 	 else {
 	 	print_r($val);
@@ -848,7 +862,7 @@ END Adodb;
 	}
 	
 	print "<p>FETCH_MODE = ASSOC: Should get 1, Caroline</p>";
-	$rs = &$db->SelectLimit('select id,firstname from ADOXYZ order by id',2);
+	$rs = $db->SelectLimit('select id,firstname from ADOXYZ order by id',2);
 	if ($rs && !$rs->EOF) {
 		if (ADODB_ASSOC_CASE == 2) {
 			$id = 'ID';
@@ -871,7 +885,7 @@ END Adodb;
 	
 	$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
 	print "<p>FETCH_MODE = NUM: Should get 1, Caroline</p>";
-	$rs = &$db->SelectLimit('select id,firstname from ADOXYZ order by id',1);
+	$rs = $db->SelectLimit('select id,firstname from ADOXYZ order by id',1);
 	if ($rs && !$rs->EOF) {
 		if (isset($rs->fields['id'])) Err("FETCH_NUM has ASSOC fields");
 		if ($rs->fields[0] != 1)  {Err("Error 1"); print_r($rs->fields);};
@@ -884,9 +898,9 @@ END Adodb;
 	
 	$db->debug = false;
 	print "<p>GetRowAssoc Upper: Should get 1, Caroline</p>";
-	$rs = &$db->SelectLimit('select id,firstname from ADOXYZ order by id',1);
+	$rs = $db->SelectLimit('select id,firstname from ADOXYZ order by id',1);
 	if ($rs && !$rs->EOF) {
-		$arr = &$rs->GetRowAssoc();
+		$arr = $rs->GetRowAssoc();
 		if ($arr['ID'] != 1) {Err("Error 1");print_r($arr);};
 		if (trim($arr['FIRSTNAME']) != 'Caroline') {Err("Error 2"); print_r($arr);};
 		$rs->MoveNext();
@@ -894,16 +908,16 @@ END Adodb;
 
 	}
 	print "<p>GetRowAssoc Lower: Should get 1, Caroline</p>";
-	$rs = &$db->SelectLimit('select id,firstname from ADOXYZ order by id',1);
+	$rs = $db->SelectLimit('select id,firstname from ADOXYZ order by id',1);
 	if ($rs && !$rs->EOF) {
-		$arr = &$rs->GetRowAssoc(false);
+		$arr = $rs->GetRowAssoc(false);
 		if ($arr['id'] != 1) {Err("Error 1"); print_r($arr);};
 		if (trim($arr['firstname']) != 'Caroline') {Err("Error 2"); print_r($arr);};
 
 	}
 	
 	print "<p>GetCol Test</p>";
-	$col = $db->GetCol('select distinct firstname from adoxyz order by 1');
+	$col = $db->GetCol('select distinct firstname from ADOXYZ order by 1');
 	if (!is_array($col)) Err("Col size is wrong");
 	if (trim($col[0]) != 'Alan' or trim($col[9]) != 'Yat Sun') Err("Col elements wrong");
 
@@ -920,7 +934,7 @@ END Adodb;
 	
 	
 	print "<p>SelectLimit Distinct Test 1: Should see Caroline, John and Mary</p>";
-	$rs = &$db->SelectLimit('select distinct * from ADOXYZ order by id',3);
+	$rs = $db->SelectLimit('select distinct * from ADOXYZ order by id',3);
 	
 	
 	if ($rs && !$rs->EOF) {
@@ -936,7 +950,7 @@ END Adodb;
 	} else Err("Failed SelectLimit Test 1");
 	
 	print "<p>SelectLimit Test 2: Should see Mary, George and Mr. Alan</p>";
-	$rs = &$db->SelectLimit('select * from ADOXYZ order by id',3,2);
+	$rs = $db->SelectLimit('select * from ADOXYZ order by id',3,2);
 	if ($rs && !$rs->EOF) {
 		if (trim($rs->fields[1]) != 'Mary') Err("Error 1 - No Mary, instead: ".$rs->fields[1]);
 		$rs->MoveNext();
@@ -952,7 +966,7 @@ END Adodb;
 	print "<p>SelectLimit Test 3: Should see Wai Hun and Steven</p>";
 	$db->debug=1;
 	global $A; $A=1;
-	$rs = &$db->SelectLimit('select * from ADOXYZ order by id',-1,48);
+	$rs = $db->SelectLimit('select * from ADOXYZ order by id',-1,48);
 	$A=0;
 	if ($rs && !$rs->EOF) {
 		if (empty($rs->connection)) print "<b>Connection object missing from recordset</b></br>";
@@ -969,7 +983,7 @@ END Adodb;
 		$db->debug = false;
 	
 	
-	$rs = &$db->Execute("select * from ADOXYZ order by id");
+	$rs = $db->Execute("select * from ADOXYZ order by id");
 	print "<p>Testing Move()</p>";	
 	if (!$rs)Err( "Failed Move SELECT");
 	else {
@@ -1028,10 +1042,10 @@ END Adodb;
 	$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
 	if ($db->dataProvider == 'postgres') {
 		$sql = "select ".$db->Concat('cast(firstname as varchar)',$db->qstr(' '),'lastname')." as fullname,id,".$db->sysTimeStamp." as d from ADOXYZ";
-		$rs = &$db->Execute($sql);
+		$rs = $db->Execute($sql);
 	} else {
 		$sql = "select distinct ".$db->Concat('firstname',$db->qstr(' '),'lastname')." as fullname,id,".$db->sysTimeStamp." as d from ADOXYZ";
-		$rs = &$db->Execute($sql);
+		$rs = $db->Execute($sql);
 	}
 	if ($rs) {
 		if (empty($_GET['hide'])) rs2html($rs);
@@ -1042,9 +1056,9 @@ END Adodb;
 	print "<hr />Testing GetArray() ";
 	//$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
 	
-	$rs = &$db->Execute("select * from ADOXYZ order by id");
+	$rs = $db->Execute("select * from ADOXYZ order by id");
 	if ($rs) {
-		$arr = &$rs->GetArray(10);
+		$arr = $rs->GetArray(10);
 		if (sizeof($arr) != 10 || trim($arr[1][1]) != 'John' || trim($arr[1][2]) != 'Lim') print $arr[1][1].' '.$arr[1][2]."<b> &nbsp; ERROR</b><br>";
 		else print " OK<BR>";
 	}
@@ -1054,7 +1068,7 @@ END Adodb;
 	echo "Testing error handling, should see illegal column 'x' error=<i>$e ($e2) </i><br>";
 	if (!$e || !$e2) Err("Error handling did not work");
 	print "Testing FetchNextObject for 1 object ";
-	$rs = &$db->Execute("select distinct lastname,firstname from ADOXYZ where firstname='Caroline'");
+	$rs = $db->Execute("select distinct lastname,firstname from ADOXYZ where firstname='Caroline'");
 	$fcnt = 0;
 	if ($rs)
 	while ($o = $rs->FetchNextObject()) {
@@ -1077,7 +1091,7 @@ END Adodb;
 	//$arr = $db->GetArray("select  lastname,firstname from ADOXYZ");
 	//print_r($arr);
 	print "<hr />";
-	$rs =& $db->Execute("select distinct lastname,firstname,created from ADOXYZ");
+	$rs = $db->Execute("select distinct lastname,firstname,created from ADOXYZ");
 	
 	if ($rs) {
 		$arr = $rs->GetAssoc();
@@ -1086,7 +1100,7 @@ END Adodb;
 		else print " OK 1";
 	}
 	
-	$arr = &$db->GetAssoc("select distinct lastname,firstname from ADOXYZ");
+	$arr = $db->GetAssoc("select distinct lastname,firstname from ADOXYZ");
 	if ($arr) {
 		//print_r($arr);
 		if (empty($arr['See']) || trim($arr['See']) != 'Wai Hun') print $arr['See']." &nbsp; <b>ERROR</b><br>";
@@ -1102,24 +1116,31 @@ END Adodb;
 	for ($loop=0; $loop < 1; $loop++) {
 	print "Testing GetMenu() and CacheExecute<BR>";
 	$db->debug = true;
-	$rs = &$db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
 	
 	
 
 
 	if ($rs) print 'With blanks, Steven selected:'. $rs->GetMenu('menu','Steven').'<BR>'; 
 	else print " Fail<BR>";
-	$rs = &$db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
 	
 	if ($rs) print ' No blanks, Steven selected: '. $rs->GetMenu('menu','Steven',false).'<BR>';
 	else print " Fail<BR>";
 	
-	$rs = &$db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
+	
+	if ($rs) print ' 1st line set to **** , Steven selected: '. $rs->GetMenu('menu','Steven','1st:****').'<BR>';
+	else print " Fail<BR>";
+	
+
+	
+	$rs = $db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
 	if ($rs) print ' Multiple, Alan selected: '. $rs->GetMenu('menu','Alan',false,true).'<BR>';
 	else print " Fail<BR>";
 	print '</p><hr />';
 	
-	$rs = &$db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
 	if ($rs) {
 		print ' Multiple, Alan and George selected: '. $rs->GetMenu('menu',array('Alan','George'),false,true);
 		if (empty($rs->connection)) print "<b>Connection object missing from recordset</b></br>";
@@ -1133,22 +1154,22 @@ END Adodb;
 	print "<hr />";
 
 	print "Testing GetMenu2() <BR>";
-	$rs = &$db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(4,"select distinct firstname,lastname from ADOXYZ");
 	if ($rs) print 'With blanks, Steven selected:'. $rs->GetMenu2('menu',('Oey')).'<BR>'; 
 	else print " Fail<BR>";
-	$rs = &$db->CacheExecute(6,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(6,"select distinct firstname,lastname from ADOXYZ");
 	if ($rs) print ' No blanks, Steven selected: '. $rs->GetMenu2('menu',('Oey'),false).'<BR>';
 	else print " Fail<BR>";
 	}
-	echo "<h3>CacheEXecute</h3>";
+	echo "<h3>CacheExecute</h3>";
 
 	$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
-	$rs = &$db->CacheExecute(6,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(6,"select distinct firstname,lastname from ADOXYZ");
 	print_r($rs->fields); echo $rs->fetchMode;echo "<br>";
 	echo $rs->Fields('firstname');
 	
 	$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
-	$rs = &$db->CacheExecute(6,"select distinct firstname,lastname from ADOXYZ");
+	$rs = $db->CacheExecute(6,"select distinct firstname,lastname from ADOXYZ");
 	print_r($rs->fields);echo "<br>";
 	echo $rs->Fields('firstname');
 	$db->debug = false;
@@ -1158,26 +1179,26 @@ END Adodb;
 	
 	$sql = 'select * from ADOXYZ where 0=1';
 	echo "<p>**Testing '$sql' (phplens compat 1)</p>";
-	$rs = &$db->Execute($sql);
+	$rs = $db->Execute($sql);
 	if (!$rs) err( "<b>No recordset returned for '$sql'</b>");
 	if (!$rs->FieldCount()) err( "<b>No fields returned for $sql</b>");
 	if (!$rs->FetchField(1)) err( "<b>FetchField failed for $sql</b>");
 	
 	$sql = 'select * from ADOXYZ order by 1';
 	echo "<p>**Testing '$sql' (phplens compat 2)</p>";
-	$rs = &$db->Execute($sql);
+	$rs = $db->Execute($sql);
 	if (!$rs) err( "<b>No recordset returned for '$sql'<br>".$db->ErrorMsg()."</b>");
 	
 	
 	$sql = 'select * from ADOXYZ order by 1,1';
 	echo "<p>**Testing '$sql' (phplens compat 3)</p>";
-	$rs = &$db->Execute($sql);
+	$rs = $db->Execute($sql);
 	if (!$rs) err( "<b>No recordset returned for '$sql'<br>".$db->ErrorMsg()."</b>");
 	
 	
 	// Move
-	$rs1 = &$db->Execute("select id from ADOXYZ where id <= 2 order by 1");
-	$rs2 = &$db->Execute("select id from ADOXYZ where id = 3 or id = 4 order by 1");
+	$rs1 = $db->Execute("select id from ADOXYZ where id <= 2 order by 1");
+	$rs2 = $db->Execute("select id from ADOXYZ where id = 3 or id = 4 order by 1");
 
 	if ($rs1) $rs1->MoveLast();
 	if ($rs2) $rs2->MoveLast();
@@ -1207,7 +1228,7 @@ END Adodb;
 	echo "<p>";
 	
 	if (substr($db->dataProvider,0,3) != 'notused') { // used to crash ado
-		$sql = "select firstnames from adoxyz";
+		$sql = "select firstnames from ADOXYZ";
 		print "<p>Testing execution of illegal statement: <i>$sql</i></p>";
 		if ($db->Execute($sql) === false) {
 			print "<p>This returns the following ErrorMsg(): <i>".$db->ErrorMsg()."</i> and ErrorNo(): ".$db->ErrorNo().'</p>';
@@ -1218,7 +1239,7 @@ END Adodb;
 	
 	print "<p>ASSOC TEST 2<br>";
 	$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
-	$rs = $db->query('select * from adoxyz order by id');
+	$rs = $db->query('select * from ADOXYZ order by id');
 	if ($ee = $db->ErrorMsg()) {
 		Err("Error message=$ee");
 	}
@@ -1239,7 +1260,7 @@ END Adodb;
 		print "<b>ADODB_FETCH_BOTH not supported</b> for dataProvider=".$db->dataProvider."<br>";
 	} else {
 		$ADODB_FETCH_MODE = ADODB_FETCH_BOTH;
-		$rs = $db->query('select * from adoxyz order by id');
+		$rs = $db->query('select * from ADOXYZ order by id');
 		for($i=0;$i<$rs->FieldCount();$i++) 
 		{ 
 			$fld=$rs->FetchField($i); 
@@ -1250,7 +1271,7 @@ END Adodb;
 	
 	print "<p>NUM TEST 2<br>";
 	$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
-	$rs = $db->query('select * from adoxyz order by id');
+	$rs = $db->query('select * from ADOXYZ order by id');
 	for($i=0;$i<$rs->FieldCount();$i++) 
 	{ 
 		$fld=$rs->FetchField($i); 
@@ -1260,7 +1281,7 @@ END Adodb;
 	
 	print "<p>ASSOC Test of SelectLimit<br>";
 	$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
-	$rs = $db->selectlimit('select * from adoxyz order by id',3,4);
+	$rs = $db->selectlimit('select * from ADOXYZ order by id',3,4);
 	$cnt = 0;
 	while ($rs && !$rs->EOF) {
 		$cnt += 1;
@@ -1277,7 +1298,7 @@ END Adodb;
 	if ($db->sysDate) {
 		$saved = $db->debug;
 		$db->debug = 1;
-		$rs = $db->Execute("select {$db->sysDate} from adoxyz where id=1");
+		$rs = $db->Execute("select {$db->sysDate} from ADOXYZ where id=1");
 		if (ADORecordSet::UnixDate(date('Y-m-d')) != $rs->UnixDate($rs->fields[0])) {
 			print "<p><b>Invalid date {$rs->fields[0]}</b></p>";
 		} else
@@ -1293,13 +1314,13 @@ END Adodb;
 	print "<p>Test CSV</p>";
 	include_once('../toexport.inc.php');
 	//$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
-	$rs = $db->SelectLimit('select id,firstname,lastname,created,\'He, he\' he,\'"\' q  from adoxyz',10);	
+	$rs = $db->SelectLimit('select id,firstname,lastname,created,\'He, he\' he,\'"\' q  from ADOXYZ',10);	
 	
 	print "<pre>";
 	print rs2csv($rs);
 	print "</pre>";
 	
-	$rs = $db->SelectLimit('select id,firstname,lastname,created,\'The	"young man", he said\' from adoxyz',10);	
+	$rs = $db->SelectLimit('select id,firstname,lastname,created,\'The	"young man", he said\' from ADOXYZ',10);	
 	
 	if (PHP_VERSION < 5) {
 		print "<pre>";
@@ -1320,13 +1341,16 @@ END Adodb;
 	$date = $db->SQLDate('d-m-M-Y-\QQ h:i:s A',$db->DBDate("1974-02-25"));
 	$sql = "SELECT $date from ADOXYZ";
 	print "<p>Test SQLDate: ".htmlspecialchars($sql)."</p>";
+	$db->debug=1;
 	$rs = $db->SelectLimit($sql,1);
 	$ts = ADOConnection::UnixDate('1974-02-25');
 	$d = date('d-m-M-Y-',$ts).'Q'.(ceil(date('m',$ts)/3.0)).date(' h:i:s A',$ts);
 	if (!$rs) {
 		Err("SQLDate query returned no recordset");
 		echo $db->ErrorMsg(),'<br>';
-	} else if ($d != $rs->fields[0]) Err("SQLDate 2 failed expected: <br>act:$d <br>sql:".$rs->fields[0]);
+	} else if ($d != reset($rs->fields)) {
+		Err("SQLDate 2 failed expected: <br>act:$d <br>sql:".$rs->fields[0].' <br>'.$db->ErrorMsg());
+	}
 	
 	
 	print "<p>Test Filter</p>";
@@ -1347,7 +1371,7 @@ END Adodb;
 	
 	print "<p>Test Replace</p>";
 	
-	$ret = $db->Replace('adoxyz', 
+	$ret = $db->Replace('ADOXYZ', 
 		array('id'=>1,'firstname'=>'Caroline','lastname'=>'Miranda'),
 		array('id'),
 		$autoq = true);
@@ -1366,14 +1390,14 @@ END Adodb;
 		}
 		$ADODB_COUNTRECS = $savec;
 	}
-	$ret = $db->Replace('adoxyz', 
+	$ret = $db->Replace('ADOXYZ', 
 		array('id'=>1000,'firstname'=>'Harun','lastname'=>'Al-Rashid'),
 		array('id','firstname'),
 		$autoq = true);
 	if ($ret != 2) print "<b>Replace failed: </b>";
 	print "test A return value=$ret (2 expected) <p>";
 	
-	$ret = $db->Replace('adoxyz', 
+	$ret = $db->Replace('ADOXYZ', 
 		array('id'=>1000,'firstname'=>'Sherazade','lastname'=>'Al-Rashid'),
 		'id',
 		$autoq = true);
@@ -1384,7 +1408,7 @@ END Adodb;
 	
 	print "<h3>rs2rs Test</h3>";
 	
-	$rs = $db->Execute('select * from adoxyz where id>= 1 order by id');
+	$rs = $db->Execute('select * from ADOXYZ where id>= 1 order by id');
 	$rs = $db->_rs2rs($rs);
 	$rs->valueX = 'X';
 	$rs->MoveNext();
@@ -1398,7 +1422,7 @@ END Adodb;
 	$db->debug=true;
  	$sql = PivotTableSQL(
  		$db,  			# adodb connection
- 		'adoxyz',  		# tables
+ 		'ADOXYZ',  		# tables
 		'firstname',	# row fields
 		'lastname',		# column fields 
 		false,			# join
@@ -1419,7 +1443,7 @@ END Adodb;
 	$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
 	
 	include_once "PEAR.php";
-	$rs = $db->query('select * from adoxyz where id>0 and id<10 order by id');
+	$rs = $db->query('select * from ADOXYZ where id>0 and id<10 order by id');
 	
 	$i = 0;
 	if ($rs && !$rs->EOF) {
@@ -1437,12 +1461,12 @@ END Adodb;
 	}
 	
 	
-	if ($i != $db->GetOne('select count(*) from adoxyz where id>0 and id<10')) {
+	if ($i != $db->GetOne('select count(*) from ADOXYZ where id>0 and id<10')) {
 		print "<p><b>PEAR DB emulation error 1.1 EOF ($i)</b></p>";
 		$pear = false;
 	}
 	
-	$rs = $db->limitQuery('select * from adoxyz where id>0 order by id',$i=3,$top=3);
+	$rs = $db->limitQuery('select * from ADOXYZ where id>0 order by id',$i=3,$top=3);
 	$i2 = $i;
 	if ($rs && !$rs->EOF) {
 
@@ -1468,7 +1492,7 @@ END Adodb;
 	flush();
 
 	
-	$rs = $db->SelectLimit("select ".$db->sysDate." from adoxyz",1);
+	$rs = $db->SelectLimit("select ".$db->sysDate." from ADOXYZ",1);
 	$date = $rs->fields[0];
 	if (!$date) Err("Bad sysDate");
 	else {
@@ -1478,9 +1502,9 @@ END Adodb;
 	}
 	$db->debug=1;
 	if ($db->dataProvider == 'oci8') 
-		$rs = $db->SelectLimit("select to_char(".$db->sysTimeStamp.",'YYYY-MM-DD HH24:MI:SS') from adoxyz",1);
+		$rs = $db->SelectLimit("select to_char(".$db->sysTimeStamp.",'YYYY-MM-DD HH24:MI:SS') from ADOXYZ",1);
 	else 
-		$rs = $db->SelectLimit("select ".$db->sysTimeStamp." from adoxyz",1);
+		$rs = $db->SelectLimit("select ".$db->sysTimeStamp." from ADOXYZ",1);
 	$date = $rs->fields[0];
 	if (!$date) Err("Bad sysTimeStamp");
 	else {
@@ -1497,9 +1521,11 @@ END Adodb;
 	flush();
 	
 	if ($db->hasTransactions) {
-		//$db->debug=1;
+		$db->debug=1;
 		echo "<p>Testing StartTrans CompleteTrans</p>";
 		$db->raiseErrorFn = false;
+		
+		$db->SetTransactionMode('SERIALIZABLE');
 		$db->StartTrans();
 		$rs = $db->Execute('select * from notable');
 			$db->StartTrans();
@@ -1508,6 +1534,8 @@ END Adodb;
 				$db->CommitTrans();
 			$db->CompleteTrans();
 		$rez = $db->CompleteTrans();
+		$db->SetTransactionMode('');
+		$db->debug=0;
 		if ($rez !== false) {
 			if (is_null($rez)) Err("Error: _transOK not modified");
 			else Err("Error: CompleteTrans (1) should have failed");
@@ -1549,7 +1577,7 @@ END Adodb;
 	if ($metae !== DB_ERROR_NOSUCHTABLE) print "<p><b>MetaError=".$metae." wrong</b>, should be ".DB_ERROR_NOSUCHTABLE."</p>";
 	else print "<p>MetaError ok (".DB_ERROR_NOSUCHTABLE."): ".$db->MetaErrorMsg($metae)."</p>";
 	if ($TESTERRS != 1) print "<b>raiseErrorFn select nowhere failed</b><br>";
-	$rs = $db->Execute('select * from adoxyz');
+	$rs = $db->Execute('select * from ADOXYZ');
 	if ($debugerr) print " Move";
 	$rs->Move(100);
 	$rs->_queryID = false;
@@ -1562,17 +1590,17 @@ END Adodb;
 	
 	print "<p>SetFetchMode() tests</p>";
 	$db->SetFetchMode(ADODB_FETCH_ASSOC);
-	$rs = $db->SelectLimit('select firstname from adoxyz',1);
+	$rs = $db->SelectLimit('select firstname from ADOXYZ',1);
 	if (!isset($rs->fields['firstname'])) Err("BAD FETCH ASSOC");
 	
 	$ADODB_FETCH_MODE = ADODB_FETCH_NUM;	
-	$rs = $db->SelectLimit('select firstname from adoxyz',1);
+	$rs = $db->SelectLimit('select firstname from ADOXYZ',1);
 	//var_dump($rs->fields);
 	if (!isset($rs->fields['firstname'])) Err("BAD FETCH ASSOC");
 	
 	$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;	
 	$db->SetFetchMode(ADODB_FETCH_NUM);
-	$rs = $db->SelectLimit('select firstname from adoxyz',1);
+	$rs = $db->SelectLimit('select firstname from ADOXYZ',1);
 	if (!isset($rs->fields[0])) Err("BAD FETCH NUM");
 	
 	flush();
@@ -1701,15 +1729,18 @@ Test <a href=test4.php>GetInsertSQL/GetUpdateSQL</a> &nbsp;
 	<a href=testpaging.php>Paging</a> &nbsp;
 	<a href=test-perf.php>Perf Monitor</a><p>
 <?php
-include('./testdatabases.inc.php');
-
-echo "<br>vers=",ADOConnection::Version();
 
 
 include_once('../adodb-time.inc.php');
 if (isset($_GET['time'])) adodb_date_test();
 
+include('./testdatabases.inc.php');
+
+echo "<br>vers=",ADOConnection::Version();
+
+
+
 ?>
-<p><i>ADODB Database Library  (c) 2000-2006 John Lim. All rights reserved. Released under BSD and LGPL, PHP <?php echo PHP_VERSION ?>.</i></p>
+<p><i>ADODB Database Library  (c) 2000-2007 John Lim. All rights reserved. Released under BSD and LGPL, PHP <?php echo PHP_VERSION ?>.</i></p>
 </body>
 </html>
